@@ -21,3 +21,8 @@ crew_test_sleep <- function() {
 expect_crew_error <- function(object) {
   testthat::expect_error(object, class = "crew_error")
 }
+
+crew_private <- function(object) {
+  skip_on_cran()
+  object$.__enclos_env__$private
+}
