@@ -45,7 +45,7 @@ crew_test("crew_launcher_local() can run a task on a worker", {
   expect_equal(launcher$workers$launches, c(0L, 1L, 0L, 0L))
   m <- mirai::mirai(ps::ps_pid(), .compute = client$name)
   crew_retry(
-    ~!.unresolved(m),
+    ~!unresolved(m),
     seconds_interval = 0.5,
     seconds_timeout = 10
   )
@@ -53,10 +53,7 @@ crew_test("crew_launcher_local() can run a task on a worker", {
   client$terminate()
   tryCatch(
     crew::crew_retry(
-      ~{
-        handle <- launcher$workers$handle[[2L]]
-        is_crew_null(handle) || !handle$is_alive()
-      },
+      ~!launcher$workers$handle[[2L]]$is_alive(),
       seconds_interval = 0.1,
       seconds_timeout = 5
     ),
@@ -100,16 +97,13 @@ crew_test("crew_launcher_local() can run a task and time out a worker", {
   )
   m <- mirai::mirai(ps::ps_pid(), .compute = client$name)
   crew::crew_retry(
-    ~!.unresolved(m),
+    ~!unresolved(m),
     seconds_interval = 0.1,
     seconds_timeout = 5
   )
   expect_equal(m$data, launcher$workers$handle[[1]]$get_pid())
   crew::crew_retry(
-    ~{
-      handle <- launcher$workers$handle[[1]]
-      is_crew_null(handle) || !handle$is_alive()
-    },
+    ~!launcher$workers$handle[[1]]$is_alive(),
     seconds_interval = 0.1,
     seconds_timeout = 5
   )
@@ -150,7 +144,9 @@ crew_test("crew_launcher_local() can run a task and end a worker", {
   client$start()
   socket <- rownames(client$daemons)
   launcher$start()
+  expect_true(launcher$workers$terminated[1L])
   launcher$launch(index = 1L)
+  expect_false(launcher$workers$terminated[1L])
   crew::crew_retry(
     ~{
       daemons <- rlang::duplicate(
@@ -164,14 +160,16 @@ crew_test("crew_launcher_local() can run a task and end a worker", {
     seconds_timeout = 10
   )
   crew::crew_retry(
-    ~{
-      handle <- launcher$workers$handle[[1]]
-      !is_crew_null(handle) && handle$is_alive()
-    },
+    ~launcher$workers$handle[[1L]]$is_alive(),
     seconds_interval = 0.1,
     seconds_timeout = 5
   )
+  expect_false(launcher$workers$terminated[1L])
+  pid <- launcher$workers$handle[[1L]]$get_pid()
+  expect_s3_class(launcher$workers$handle[[1L]], "process")
   expect_silent(launcher$terminate())
+  expect_s3_class(launcher$workers$handle[[1L]], "process")
+  expect_true(launcher$workers$terminated[1L])
   crew::crew_retry(
     ~{
       daemons <- rlang::duplicate(
@@ -185,11 +183,16 @@ crew_test("crew_launcher_local() can run a task and end a worker", {
     seconds_timeout = 10
   )
   crew::crew_retry(
-    ~{
-      handle <- launcher$workers$handle[[1]]
-      is_crew_null(handle) || !handle$is_alive()
-    },
+    ~!launcher$workers$handle[[1]]$is_alive(),
     seconds_interval = 0.1,
     seconds_timeout = 5
   )
+  expect_equal(launcher$workers$termination[[1L]]$pid, pid)
+  expect_silent(launcher$terminate())
+  expect_true(launcher$workers$terminated[1L])
+})
+
+crew_test("deprecate seconds_exit", {
+  suppressWarnings(crew_launcher_local(seconds_exit = 1))
+  expect_true(TRUE)
 })
